#!/bin/bash

# Copyright (C) 2013, Moritz Duge, esist[at]kolahilft[dot]de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Here is where you can get a copy of the GPL version 2.
# https://www.gnu.org/licenses/gpl-2.0.html


# Removes last ip-octet from nginx access and error log

# Example: loganonymizer.bash -5760 /var/log/nginx/
# This will anonymize all error.log* and access.log* in
# that directory, that where modified since 4*24*60
# miniutes (4 days) and contain non-anonymized ips.
# Use +0 as time, to ignore modification time.


mins="${1}"
dir="${2}"

usage='Usage: loganonymizer.bash -5760 /var/log/nginx/'
if [ "${mins}" == "" ]; then echo "${usage}"; exit 1; fi
if [ "${dir}" == "" ]; then echo "${usage}"; exit 1; fi

access_grep='^([0-9]{1,3}\.){3}([1-9]|[0-9]{2,3}) '
access_sed='s/^\(\([0-9]*\.\)*\)[0-9]* /\10 /g'

for i in $(find "${dir}" -cmin "${mins}" -name 'access.log*'); do
	echo -n 'Checking '"${i}"' ...'
	
	suffix="${i##*.}"
	if [ "${suffix}" == 'gz' ]; then
		dirty=$(zgrep -lP "${access_grep}" "${i}")
		if [ "${dirty}" == "${i}" ]; then
			tmp=$(zcat "${i}" | sed -e "${access_sed}")
			echo -n "${tmp}" | gzip > "${i}"
			echo 'done'
		else
			echo 'skipped'
		fi
	else
		dirty=$(grep -lP "${access_grep}" "${i}")
		if [ "${dirty}" == "${i}" ]; then
			sed -i -e "${access_sed}" "${i}"
			echo 'done'
		else
			echo 'skipped'
		fi
	fi
done


error_grep=', client: ([0-9]{1,3}\.){3}([1-9]|[0-9]{2,3}), '
error_sed='s/, client: \(\([0-9]*\.\)*\)[0-9]*, /, client: \10, /g'

for i in $(find "${dir}" -cmin "${mins}" -name 'error.log*'); do
	echo -n 'Checking '"${i}"' ...'
	
	suffix="${i##*.}"
	if [ "${suffix}" == 'gz' ]; then
		dirty=$(zgrep -lP "${error_grep}" "${i}")
		if [ "${dirty}" == "${i}" ]; then
			tmp=$(zcat "${i}" | sed -e "${error_sed}")
			echo -n "${tmp}" | gzip > "${i}"
			echo 'done'
		else
			echo 'skipped'
		fi
	else
		dirty=$(grep -lP "${error_grep}" "${i}")
		if [ "${dirty}" == "${i}" ]; then
			sed -i -e "${error_sed}" "${i}"
			echo 'done'
		else
			echo 'skipped'
		fi
	fi
done

